#!/usr/bin/perl

package XYCSV;

sub new
{
	my $class = shift;
	my $initfile = shift;
	my $self = {};
	bless $self, $class;
	$self->{Header} = [];

	if(defined $initfile){ $self->ReadData($initfile); }

	return $self;
}

my $zahl = '\+?\-?\d*\.?\d+e?E?\+?\-?\d*';
my $trenner = '\s+|,|;';

sub ReadData #(file)
{
	my $self = shift;
	%{$self->{Data}} = (); #initialisation
	my $f = shift;
	$self->ReadHeader($f) if ($#{$self->{Header}} == -1);
	
	if(open(LD, $f))
	{
		while(<LD>)
		{
			if($_ =~ /^($zahl)($trenner)($zahl)/)
			{
				#print "read $1 : $3\n";
				push(@{$self->{Data}->{$1}}, $3);
			}
		}
		close(LD);
		return 0;
	}
	else
	{
		print STDERR "Konnte $f nicht ffnen ($!)!\n";
		return -1;
	}
}

sub WriteData #(file)
{
	my $self = shift;
	my $f = shift;
	unless(defined $f){ $wld = STDOUT; }
	else{ open(WLD, ">$f") or die "Cannot open $f for writing! ($!)\n"; $wld = WLD;}
	
	print $wld ${$self->RawHeader()};
	for( sort { $a <=> $b } keys %{$self->{Data}} )
	{
		print $wld $_,"\t",join("\n$_\t", @{$self->{Data}->{$_}} ), "\n";
	}
	if($wld ne STDOUT){close(WLD);}
	return 0;
}

sub OnePointPerInterval #(interval width)
{
	my $self = shift;
	my $int = shift;
	my @inty = ();
	my %newdata = ();
	my $i = 0;
	my $firstx = undef;
	for( sort { $a <=> $b } keys %{$self->{Data}} )
	{
		my $x = $_;
		unless(defined $firstx)
		{
			$firstx = $x;
			if( $firstx < 0 )
			{
				while(((--$i-1)*$int) > $firstx)
				{
					#print "first downcount: ",$i*$int,"($firstx)\n";
				}
			}
			else
			{
				while(((++$i)*$int) < $firstx)
				{
					#print "first upcount: ",$i*$int,"($firstx)\n";
				}
			}
		}
		if($x >= $i*$int) # [start,end) ... maybe one could share a border between intervals
		{
			push(@inty, @{$self->{Data}->{$x}}) if $x == $i*$int; #take the ending border into account here
			if($#inty > -1)
			{
				#set data point
				@{$newdata{($i-0.5)*$int}} = ($self->Mean(\@inty)); 
			}
			while(((++$i)*$int) < $x)
			{
				#print "upcount: ",$i*$int,"($x)\n";
			}
			@inty = ();
		}
		
		push(@inty,@{$self->{Data}->{$x}}); #remember all points associated with this x, old ending border may become new start border here
		
	} 
	$self->{Data} = \%newdata;
	return 0;
}


sub ReadHeader #(file)
{
	my $self = shift;
	my $f = shift;
	$self->{Header} = [];
	
	if(open(LD, $f))
	{
		while(<LD>)
		{
			#why does the # get duplicated??? Not solving now, but this is _very_ strange!
			if($_ =~ /^#*([^\d\+\-].*)$/){ push(@{$self->{Header}}, $1); }
			else{ last; }
		}
		close(LD);
		return 0;
	}
	else
	{
		print STDERR "Konnte $f nicht ffnen ($!)!\n";
		return -1;
	}
}

sub RawHeader
{
	my $self = shift;
	my $hs = '';
	for(@{$self->{Header}})
	{
		$hs .= '#'.$_."\n";
	}
	return \$hs;	
}

sub HeaderData #gets title and axes, if possible...
{
	my $self = shift;
	my %hd = ('Title','','X','','Y','');
	my $i = $#{$self->{Header}};
	my $ax = $i;
	if( $i >= 0)
	{
		if( $i > 0)
		{
			#that's great! we'll be not too cautionous
			if($self->{Header}->[0] =~ /^"?(.+)"?$/){ $hd{Title} = $1; }
		}
		else
		{
			$ax = 0;
		}
		#in every case: guess axes
		if($self->{Header}->[$ax] =~ /^"?([^"]+)"?($trenner)"?([^"]+)"?$/)
		{
			$hd{X} = $1; $hd{Y} = $3;
		} 
	}
	return \%hd;
}

sub Y
{
	my $self=shift;
	my $y = shift;
	if(defined $self->{Data}->{$y}){ return $self->{Data}->{$y}->[0]; }
	else{ return undef; }
}

sub YValuesInXRange
{
	my $self=shift;
	my $start = shift;
	my $end = shift;
	my @list = ();
	if($start > $end){my $swp = $start; $start = $end; $end = $swp;}
	for( sort { $a <=> $b } keys %{$self->{Data}} )
	{
		if( $_ >= $start && $_ <= $end)
		{
			push(@list, @{$self->{Data}->{$_}});
		}
		if( $_ > $end){ last; }
	}
	return \@list;
}

sub Mean
{
	my $self = shift;
	my $list = shift;
	#calculate arithmetic mid
	my $y = 0;
	if($#{$list} == -1){return undef;}
	if($#{$list} == 0){return $list->[0];}
	for(@{$list})
	{
		#print "Val: $_\n";
		$y += $_;
	}
	return $y / ($#{$list}+1);
}
